#version 110

#ifdef GL_ES
precision mediump float;
#endif

uniform vec3      iResolution;           // viewport resolution (in pixels)
uniform float     iDuration;             // shader playback total time (in seconds)
uniform float     iTime;                 // shader playback current time (in seconds)
uniform float     iTimeDelta;            // render time (in seconds)
uniform int       iFrame;                // shader playback frame
uniform float     iChannelTime[8];       // channel playback time (in seconds)
uniform vec3      iChannelResolution[8]; // channel resolution (in pixels)
uniform vec4      iMouse;                // mouse pixel coords. xy: current (if MLB down), zw: click
uniform sampler2D iChannel[8];           // input channel. XX = 2D/Cube
uniform vec4      iDate;                 // (year, month, day, time in seconds)
uniform float     iSampleRate;           // sound sample rate (i.e., 44100)

uniform float     strength;


float iProgress = iTime / iDuration;

const float PI = 3.141592653589793;

////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
float reflection = 0.4;
float perspective = 0.2;
float depth = 3.0;
 
const vec4 black = vec4(0.0, 0.0, 0.0, 1.0);
const vec2 boundMin = vec2(0.0, 0.0);
const vec2 boundMax = vec2(1.0, 1.0);
 
bool inBounds (vec2 p) {
  return all(lessThan(boundMin, p)) && all(lessThan(p, boundMax));
}
 
vec2 project (vec2 p) {
  return p * vec2(1.0, -1.2) + vec2(0.0, -0.02);
}
 
vec4 bgColor (vec2 p, vec2 pfr, vec2 pto) {
  vec4 c = black;
  pfr = project(pfr);
  if (inBounds(pfr)) {
    c += mix(black, texture2D(iChannel[0], pfr), reflection * mix(1.0, 0.0, pfr.y));
	if(pfr.y <= 0.01)
    {
      float a = smoothstep(0., 0.01, pfr.y);
	  c = mix(vec4(vec3(0.0), 1.0), c, a);
    }
  }
  pto = project(pto);
  if (inBounds(pto)) {
    c += mix(black, texture2D(iChannel[1], pto), reflection * mix(1.0, 0.0, pto.y));
	if(pto.y <= 0.01)
    {
      float a = smoothstep(0., 0.01, pto.y);
	  c = mix(vec4(vec3(0.0), 1.0), c, a);
    }
  }
  return c;
}
 
void mainImage( out vec4 fragColor, in vec2 fragCoord ) {
  vec2 p = fragCoord / iResolution.xy;
 
  vec2 pfr, pto = vec2(-1.);
 
  float size = mix(1.0, depth, iProgress);
  float persp = perspective * iProgress;
  pfr = (p + vec2(-0.0, -0.5)) * vec2(size/(1.0-perspective*iProgress), size/(1.0-size*persp*p.x)) + vec2(0.0, 0.5);
 
  size = mix(1.0, depth, 1.-iProgress);
  persp = perspective * (1.-iProgress);
  pto = (p + vec2(-1.0, -0.5)) * vec2(size/(1.0-perspective*(1.0-iProgress)), size/(1.0-size*persp*(0.5-p.x))) + vec2(1.0, 0.5);
 
  bool fromOver = iProgress < 0.5;
 
 float a = 1.0;
  if (fromOver) {
    if (inBounds(pfr)) {
	  if (pfr.y >= 0.99)
	  {
        a = 1.0 - smoothstep(0.99, 1.0, pfr.y);
      }
      else if(pfr.y <= 0.01)
      {
        a = smoothstep(0., 0.01, pfr.y);
      }
      fragColor = mix(vec4(vec3(0.0), 1.0), texture2D(iChannel[0], pfr), a);
    }
    else if (inBounds(pto)) {
	  if (pto.y >= 0.99)
	  {
        a = 1.0 - smoothstep(0.99, 1.0, pto.y);
      }
      else if(pto.y <= 0.01)
      {
        a = smoothstep(0., 0.01, pto.y);
      }
      fragColor = mix(vec4(vec3(0.0), 1.0), texture2D(iChannel[1], pto), a);
    }
    else {
      fragColor = bgColor(p, pfr, pto);
    }
  }
  else {
    if (inBounds(pto)) {
      if (pto.y >= 0.99)
	  {
        a = 1.0 - smoothstep(0.99, 1.0, pto.y);
      }
      else if(pto.y <= 0.01)
      {
        a = smoothstep(0., 0.01, pto.y);
      }
      fragColor = mix(vec4(vec3(0.0), 1.0), texture2D(iChannel[1], pto), a);
    }
    else if (inBounds(pfr)) {
      if (pfr.y >= 0.99)
	  {
        a = 1.0 - smoothstep(0.99, 1.0, pfr.y);
      }
      else if(pfr.y <= 0.01)
      {
        a = smoothstep(0., 0.01, pfr.y);
      }
      fragColor = mix(vec4(vec3(0.0), 1.0), texture2D(iChannel[0], pfr), a);
    }
    else {
      fragColor = bgColor(p, pfr, pto);
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

void main() {
    mainImage(gl_FragColor, gl_FragCoord.xy);
}
